unit uSGRest;

interface

uses
  MVCFramework.RESTClient,
  System.Classes;

type

  TRestCallType = (rcGet, rcPost, rcPut, rcDelete);

  TClientRest = class
  private
    class function GetRestClient(const aAccept: String; AServer: string; APort: string): TRESTClient;
    class function MVCCall(aCallType: TRestCallType; const aServer: string; aPort: string; const aResource: String; const AResourceParams: array of string; aJSON: String = '';
  aPDF: Boolean = false; aStream: TStream = nil): String; overload;
  public
    class function AuthorizeDevice(const aCode: string; const aDeviceId: string; const aServer: string; aPort: string): string;
    class function UpdateDevice(const aDeviceId: string; const aDeviceToken: string; const aDeviceDescription: string; aDeviceOs: string; aDeviceOsVersion: string; aDeviceType: string; aDeviceModel: string; const aServer: string; aPort: string): string;
  end;

implementation

uses
  MVCFramework.SystemJSONUtils,
  //MVCFramework.TypesAliases,
  System.NetEncoding,
  System.SysUtils;

{ TClientRest }

class function TClientRest.AuthorizeDevice(const aCode: string; const aDeviceId: string; const aServer: string; aPort: string): string;
begin
  Result := MVCCall(rcPost, aServer, aPort, '/device/authorize', [aCode,aDeviceId]);
end;

class function TClientRest.GetRestClient(const aAccept: String; AServer: string; APort: string): TRESTClient;
var
  lServer: String;
  lPort: Word;
begin
  lServer := aServer;
  lPort := aPort.ToInteger();
  Result := TRESTClient.Create(lServer, lPort);
  Result.Accept(aAccept);
  Result.ConnectionTimeOut(10000);
  Result.ReadTimeOut(20000);
end;

class function TClientRest.UpdateDevice(const aDeviceId: string; const aDeviceToken: string; const aDeviceDescription: string; aDeviceOs: string; aDeviceOsVersion: string; aDeviceType: string; aDeviceModel: string; const aServer: string; aPort: string): string;
begin
  Result := MVCCall(rcPut, aServer, aPort, '/device', [aDeviceId], Format('{"devicetoken":"%s", "description" : "%s", "devicetype": "%s", "deviceos": "%s", "deviceosversion": "%s", "devicemodel":"%s"}', [aDeviceToken,aDeviceDescription,aDeviceType,aDeviceOs,aDeviceOsVersion,aDeviceModel]));
end;

class function TClientRest.MVCCall(aCallType: TRestCallType; const aServer: string; aPort: string; const aResource: String; const AResourceParams: array of string; aJSON: String = '';
  aPDF: Boolean = false; aStream: TStream = nil): String;
const
  cPDF = 'application/pdf';
  cAll = 'application/json;text/plain';
  cFile = 'application/octet-stream';
var
  lClient: TRESTClient;
  lResp: IRESTResponse;
begin
  Result := '';

  if aPDF then
    lClient := GetRestClient(cPDF, aServer, aPort)
  else if Assigned(aStream) then
    lClient := GetRestClient(cFile, aServer, aPort)
  else
    lClient := GetRestClient(cAll, aServer, aPort);

  try
    case aCallType of
      rcGet:
        lResp := lClient.doGET(aResource, AResourceParams);
      rcPost:
        lResp := lClient.doPOST(aResource, AResourceParams, aJSON);
      rcPut:
        lResp := lClient.doPUT(aResource, AResourceParams, aJSON);
      rcDelete:
        lResp := lClient.doDELETE(aResource, AResourceParams);
    end;

    if lResp.HasError then
      raise Exception.Create(lResp.Error.ExceptionMessage);

    if Assigned(aStream) then
    begin
      TMemoryStream(lResp.Body).SaveToStream(aStream);
      aStream.Position := 0;
    end
    else
      Result := lResp.BodyAsString;

  finally
    lClient.Free;
  end;
end;

end.
