unit uSGMainForm;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes,
  System.Variants, System.Notification, System.IniFiles, System.Messaging,
  FMX.Types, FMX.Graphics, FMX.Controls, FMX.Forms, FMX.Dialogs, FMX.StdCtrls,
  FMX.Controls.Presentation, FMX.ActnList, FMX.TabControl, System.Actions,
  System.IOUtils, System.Threading, FMX.Platform, FMX.VirtualKeyboard,
  FMX.ScrollBox, FMX.Memo, System.PushNotification, FMX.Edit, FMX.Layouts,
  FMX.Menus, System.JSON,
  FMX.WebBrowser, FMX.MultiView, System.Math.Vectors, FMX.Controls3D,
  FMX.Layers3D, FMX.Objects
{$IFDEF Android}
   ,Androidapi.JNI.Os  //TJBuild
   ,Androidapi.Helpers // StringToJString
   ,FMX.PushNotification.Android
{$ENDIF}
{$IFDEF IOS}
  ,iOSapi.UIKit
  ,Posix.SysSysctl
  ,Posix.StdDef
{$ENDIF}
  ,uProgressFrame,
  Data.Bind.Components, Data.Bind.ObjectScope,
  uSGRest,
  REST.Backend.PushTypes,
  REST.Backend.EMSProvider, REST.Backend.BindSource, REST.Backend.PushDevice,
  REST.Backend.EMSPushDevice;

type
  TSGMainForm = class(TForm)
    Header: TToolBar;
    HeaderLabel: TLabel;
    ActionList1: TActionList;
    TitleAction: TControlAction;
    PreviousTabAction1: TPreviousTabAction;
    NextTabAction1: TNextTabAction;
    Memo1: TMemo;
    MainTabControl: TTabControl;
    TabItem1: TTabItem;
    TabItem2: TTabItem;
    TabItem3: TTabItem;
    ShowConfigurationAction: TAction;
    ShowLogMessageAction: TAction;
    RegisterTokenOnServerAction: TAction;
    Label1: TLabel;
    FlowLayout1: TFlowLayout;
    AppIdEdit: TEdit;
    ShowAboutAction: TAction;
    MenuSpeedButton: TSpeedButton;
    Layout1: TLayout;
    Button1: TButton;
    SaveConfigurationAction: TAction;
    TabItem4: TTabItem;
    Layout2: TLayout;
    Button2: TButton;
    Label2: TLabel;
    ControlCodeEdit: TEdit;
    Layout3: TLayout;
    CheckCodeAction: TAction;
    WebBrowser1: TWebBrowser;
    Label3: TLabel;
    ServerAddressEdit: TEdit;
    Label4: TLabel;
    ServerPortEdit: TEdit;
    MainMultiView: TMultiView;
    InsertCodeButton: TButton;
    Layout3D1: TLayout3D;
    LogButton: TButton;
    ConfigurationButton: TButton;
    AboutButton: TButton;
    ShowInsertCodeAction: TAction;
    ProgressFrame1: TProgressFrame;
    StyleBook1: TStyleBook;
    Image1: TImage;
    Image2: TImage;
    Image3: TImage;
    PushEvents1: TPushEvents;
    EMSProvider1: TEMSProvider;
    procedure FormShow(Sender: TObject);
    procedure ShowConfigurationActionExecute(Sender: TObject);
    procedure ShowLogMessageActionExecute(Sender: TObject);
    procedure RegisterTokenOnServerActionExecute(Sender: TObject);
    procedure ShowAboutActionExecute(Sender: TObject);
    procedure SaveConfigurationActionExecute(Sender: TObject);
    procedure CheckCodeActionExecute(Sender: TObject);
    procedure ShowInsertCodeActionExecute(Sender: TObject);
    procedure MainMultiViewShown(Sender: TObject);
    procedure MainMultiViewHidden(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; var KeyChar: Char;
      Shift: TShiftState);
    procedure ShowVirtualKeyBoard(Sender: TObject; const Point: TPointF);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure PushEvents1DeviceTokenReceived(Sender: TObject);
    procedure PushEvents1DeviceTokenRequestFailed(Sender: TObject;
      const AErrorMessage: string);
    procedure PushEvents1PushReceived(Sender: TObject; const AData: TPushData);
  private
    FShowNotification: boolean;
    FPushService: TPushService;
    FServiceConnection: TPushServiceConnection;
    FClientRest: TClientRest;
    FDeviceId: string;
    FDeviceToken: string;
    FAppId: string;
    FServerAddress: string;
    FServerPort: string;
    FOSVersion: string;
    FOSLang: string;
    FModelName: string;
    FOSPlatform: string;
    FOSMajMin: string;
    //procedure OnReceiveNotificationEvent(Sender: TObject; const ServiceNotification: TPushServiceNotification);
    //procedure OnServiceConnectionChange(Sender: TObject; PushChanges: TPushService.TChanges);
    procedure ShowNotification(AName: string; AAlertBody: string; ATitle: string; ASound: boolean; ANumber: integer);
    procedure CallOnMainThread(aMethod: TThreadProcedure);
    function AppConfigurationOk: boolean;
    {$IFDEF IOS}
    function GetDeviceModelString: String;
    {$ENDIF}
    procedure GetInfoDevice;
    procedure ApplySettingsOnEdit;
    procedure ShowActivity;
    procedure HideActivity;
    procedure SetClientRestParameters;
  public
  end;

var
  SGMainForm: TSGMainForm;

const
  cIniFileName = 'sgapp.ini';
  cIniSection = 'parameters';

implementation

{$R *.fmx}

procedure TSGMainForm.HideActivity;
begin
  ProgressFrame1.HideActivity;
  MainMultiView.Enabled := True;
end;

procedure TSGMainForm.MainMultiViewHidden(Sender: TObject);
begin
  WebBrowser1.Visible := True;
end;

procedure TSGMainForm.MainMultiViewShown(Sender: TObject);
begin
  WebBrowser1.Visible := False;
end;

procedure TSGMainForm.RegisterTokenOnServerActionExecute(Sender: TObject);
var
  LResponse: string;
  LJSONObject: TJSONObject;
  LStatusCode: integer;
  LMessage: string;
  LDeviceType: string;
begin
  GetInfoDevice;

  {$IFDEF ANDROID}
  LDeviceType := '1';
  {$ENDIF}
  {$IFDEF IOS}
  LDeviceType := '0';
  {$ENDIF}

  if (not FServerAddress.IsEmpty) and (not FServerPort.IsEmpty) then
  begin
    ShowActivity;

    ITask(TTask.Create(
      procedure
      begin
        try
          try

            LResponse := FClientRest.UpdateDevice(FDeviceId, FDeviceToken, FModelName, FOSPlatform, FOSMajMin, LDeviceType, FModelName);

            CallOnMainThread(
              procedure
              begin
                Memo1.Lines.Add('RESPONSE UpdateDevice: '+LResponse);
              end
            );

            LJSONObject := TJSONObject.ParseJSONValue(LResponse) as TJSONObject;
            try

            LJSONObject.TryGetValue<integer>('statuscode',LStatusCode);
            LJSONObject.TryGetValue<string>('message',LMessage);

            if LStatusCode<>200 then
              raise Exception.Create(Format('Error in Update Device on Server %s %s:'+LStatusCode.ToString+' '+LMessage,[FServerAddress, FServerPort]));

            finally
              LJSONObject.Free;
            end;

          except
            on E: Exception do
            begin
              CallOnMainThread(
                procedure
                begin
                  ShowMessage(E.Message);
                end);
            end;
          end;

        finally
          CallOnMainThread(
            procedure
            begin
              HideActivity;
            end);
        end;
      end)).Start;
  end
  else
    raise Exception.Create('Server Address or Server Port not set in configuration');
end;

procedure TSGMainForm.ShowAboutActionExecute(Sender: TObject);
begin
  // Chiude Multiview
  MainMultiView.HideMaster;
  ShowMessage('DELPHI DAY 2019 DEMO APP');
end;

procedure TSGMainForm.ShowActivity;
begin
  MainMultiView.Enabled := False;
  ProgressFrame1.ShowActivity;
end;

function TSGMainForm.AppConfigurationOk: boolean;
var
 Ini: TIniFile;
 LIniFileName: string;
begin
  LIniFileName := System.IOUtils.TPath.Combine(System.IOUtils.TPath.GetDocumentsPath, cIniFileName);

  if TFile.Exists(LIniFileName) then
  begin
    Ini := TIniFile.Create(LIniFileName);
    try
      AppIdEdit.Text := Ini.ReadString(cIniSection, 'AppId', '');
      ServerAddressEdit.Text := Ini.ReadString(cIniSection, 'ServerAddress', '127.0.0.1');
      ServerPortEdit.Text := Ini.ReadString(cIniSection, 'ServerPort', '8080');
    finally
      Ini.Free;
    end;
  end
  else
  begin
    AppIdEdit.Text := '';
    ServerAddressEdit.Text := '';
    ServerPortEdit.Text := '';
  end;

  if not AppIdEdit.Text.IsEmpty then
  begin
    // Salva nelle variabili globali di form
    FAppId := AppIdEdit.Text;
    FServerAddress := ServerAddressEdit.Text;
    FServerPort := ServerPortEdit.Text;;
    Result := True;
  end
  else
  begin
    // Svuota le variabili globali di form
    FAppId := '';
    FServerAddress := '';
    FServerPort := '';
    Result := False;
  end;
end;

procedure TSGMainForm.ApplySettingsOnEdit;
var
  I: Integer;
begin
  for I := 0 to Self.ComponentCount-1 do
  begin
    if Self.Components[I] is TEdit then
    begin
      (Self.Components[I] as TEdit).ReturnKeyType := TReturnKeyType.Done;
      (Self.Components[I] as TEdit).OnTap := ShowVirtualKeyBoard;
    end;
  end;
end;

procedure TSGMainForm.CallOnMainThread(aMethod: TThreadProcedure);
begin
  TThread.Synchronize(nil, aMethod);
end;

procedure TSGMainForm.CheckCodeActionExecute(Sender: TObject);
var
  LResponse: string;
  LJSONObject: TJSONObject;
  LMessage: string;
  LStatusCode: integer;
begin
  if not ControlCodeEdit.Text.IsEmpty then
  begin
    if (not FServerAddress.IsEmpty) and (not FServerPort.IsEmpty) then
    begin
      ShowActivity;

      ITask(TTask.Create(
        procedure
        begin
          try
            try
              LResponse := FClientRest.AuthorizeDevice(ControlCodeEdit.Text,FDeviceId);

              CallOnMainThread(
                procedure
                begin
                  Memo1.Lines.Add('RESPONSE AuthorizeDevice: '+LResponse);
                end
              );

              LJSONObject := TJSONObject.ParseJSONValue(LResponse) as TJSONObject;
              try

              LJSONObject.TryGetValue<integer>('statuscode',LStatusCode);
              LJSONObject.TryGetValue<string>('message',LMessage);

              finally
                LJSONObject.Free;
              end;

              CallOnMainThread(
                procedure
                begin
                  WebBrowser1.URL := LMessage;
                  MainTabControl.TabIndex := 0;
                  FShowNotification := True;
                end
              );

            except
              on E: Exception do
              begin
                CallOnMainThread(
                  procedure
                  begin
                    ShowMessage('Server Response With Errors! '+LineFeed+'Error Description:'+E.Message);
                  end
                );
              end;
            end;

          finally
            TThread.Synchronize(nil,
              procedure
              begin
                HideActivity;
              end);
          end;
        end)).Start;
    end
    else
      raise Exception.Create('Server Address or Server Port not set in configuration');
  end
  else
    raise Exception.Create('Control Code is Empty');
end;

procedure TSGMainForm.FormCreate(Sender: TObject);
var
  LPushData: TPushData;
begin
  // DAVE NOTTAGE -> serve per quando l'app  chiusa completamente e arriva la notifica
  // altrimenti non si scatena l'evento PUSHRECEIVED
  //LPushData := PushEvents1.StartupNotification;
  //if LPushData <> nil then
  //  PushEvents1PushReceived(nil, LPushData);
  // Crea oggetto per chiamate REST verso il server
  FClientRest := TClientRest.Create;
  FShowNotification := False;
end;

procedure TSGMainForm.PushEvents1DeviceTokenReceived(Sender: TObject);
begin
  FDeviceId := PushEvents1.DeviceID;
  FDeviceToken := PushEvents1.DeviceToken;

  CallOnMainThread(
    procedure
    begin
      Memo1.Lines.Add('DeviceID: ' + FDeviceId);
      {$IFDEF ANDROID}
      Memo1.Lines.Add('FCM Token: ' + FDeviceToken);
      {$ENDIF}
      {$IFDEF IOS}
      Memo1.Lines.Add('APN Token: ' + FDeviceToken);
      {$ENDIF}
      Memo1.Lines.Add('Ready to receive!');
    end
  );

  if FDeviceToken.IsEmpty then
  begin
    ShowMessage('Error in Token Recovering, Please Check Configuration');
    Exit;
  end;

  RegisterTokenOnServerAction.Execute;
end;

procedure TSGMainForm.PushEvents1DeviceTokenRequestFailed(Sender: TObject;
  const AErrorMessage: string);
begin
  CallOnMainThread(
    procedure
    begin
      ShowMessage('Device Token Request Failed');
    end
  );
end;

procedure TSGMainForm.PushEvents1PushReceived(Sender: TObject;
  const AData: TPushData);
begin
  CallOnMainThread(
    procedure
    var
     I: Integer;
    begin
      {$IFDEF ANDROID}
      for I := 0 to AData.Extras.Count-1 do
      begin
        Memo1.Lines.Add('Key: ' + AData.Extras.Items[I].Key+' Value:'+AData.Extras.Items[I].Value);
      end;
      // Se la notifica arriva quando l'app  aperta la mostra tra le notifiche
      if Assigned(Sender) then  ShowNotification('MyNot',AData.Message,AData.GCM.Title,True,0);
      {$ENDIF}
      {$IFDEF IOS}
      Memo1.Lines.Add('Alert: ' + AData.APS.Alert);
      //Memo1.Lines.Add('Title: ' + AData.APS.Title);
      //Memo1.Lines.Add('Body: ' + AData.APS.Body);
      {$ENDIF}
    end
  );
end;

procedure TSGMainForm.FormDestroy(Sender: TObject);
begin
  FClientRest.Free;
end;

procedure TSGMainForm.FormKeyDown(Sender: TObject; var Key: Word;
  var KeyChar: Char; Shift: TShiftState);
var
  KeyboardService: IFMXVirtualKeyboardService;
begin
  //if Key = vkHardwareBack then
  //begin
  //  Key := 0;
  //end;

  if Key = vkReturn then
  begin
    if TPlatformServices.Current.SupportsPlatformService(IFMXVirtualKeyboardService, IInterface(KeyboardService)) then
      KeyboardService.HideVirtualKeyboard;
  end;
end;

procedure TSGMainForm.FormShow(Sender: TObject);
begin
  ApplySettingsOnEdit;

  //if AppConfigurationOk then
  //begin
    SetClientRestParameters;
    // Si posiziona sul Tab della richiesta codice di controllo
    MainTabControl.TabIndex := 3;
  //end
  //else
  //  ShowConfigurationAction.Execute;
end;

procedure TSGMainForm.ShowLogMessageActionExecute(Sender: TObject);
begin
  // Chiude Multiview
  MainMultiView.HideMaster;
  MainTabControl.TabIndex := 2;
end;

procedure TSGMainForm.SaveConfigurationActionExecute(Sender: TObject);
var
 Ini: TIniFile;
 LIniFileName: string;
 LAppId: string;
 LServerAddress: string;
 LServerPort: string;
begin
  if AppIdEdit.Text.IsEmpty then
    raise Exception.Create('Please Insert AppId');

  if ServerAddressEdit.Text.IsEmpty then
    raise Exception.Create('Please Insert Server Address');

  if ServerPortEdit.Text.IsEmpty then
    raise Exception.Create('Please Insert Server Port');

  LIniFileName := System.IOUtils.TPath.Combine(System.IOUtils.TPath.GetDocumentsPath, cIniFileName);

  Ini := TIniFile.Create(LIniFileName);
  try
    // Salva nel file di configurazione i dati
    Ini.WriteString(cIniSection, 'AppId', AppIdEdit.Text);
    Ini.WriteString(cIniSection, 'ServerAddress', ServerAddressEdit.Text);
    Ini.WriteString(cIniSection, 'ServerPort', ServerPortEdit.Text);

    // Salva nelle variabili globali di form
    FAppId := AppIdEdit.Text;
    FServerAddress := ServerAddressEdit.Text;
    FServerPort := ServerPortEdit.Text;
  finally
    Ini.Free;
  end;

  SetClientRestParameters;
  // Si sposta sul TAB della richiesta del codice
  MainTabControl.TabIndex := 3;
  ControlCodeEdit.Text := '';
end;

procedure TSGMainForm.SetClientRestParameters;
begin
  FClientRest.Server := '82.223.55.227';
  FClientRest.Port := 8080;

  FServerAddress := '82.223.55.227';
  FServerPort := '8080';
end;

procedure TSGMainForm.ShowConfigurationActionExecute(Sender: TObject);
begin
  // Chiude Multiview
  MainMultiView.HideMaster;
  // Si sposta sul TAB della configurazione
  MainTabControl.TabIndex := 1;
end;

procedure TSGMainForm.ShowInsertCodeActionExecute(Sender: TObject);
begin
  // Chiude Multiview
  MainMultiView.HideMaster;
  // Si sposta sul TAB per l'inserimento del codice
  MainTabControl.TabIndex := 3;
  ControlCodeEdit.Text := '';
end;

procedure TSGMainForm.ShowNotification(AName: string; AAlertBody: string; ATitle: string; ASound: boolean; ANumber: integer);
var
  LNotificationCenter: TNotificationCenter;
  LNotification: TNotification;
begin
  LNotificationCenter := TNotificationCenter.Create(nil);
  try
    LNotification := LNotificationCenter.CreateNotification;
    try
      LNotification.Name := AName;
      LNotification.AlertBody := AAlertBody;
      LNotification.Title := ATitle;
      LNotification.EnableSound := ASound;
      LNotification.Number := ANumber;
      LNotificationCenter.ApplicationIconBadgeNumber := ANumber;
      LNotificationCenter.PresentNotification(LNotification);
    finally
      LNotification.DisposeOf;
    end;
  finally
    LNotificationCenter.Free;
    LNotificationCenter.DisposeOf;
  end;
end;

procedure TSGMainForm.ShowVirtualKeyBoard(Sender: TObject; const Point: TPointF);
var
  KeyboardService: IFMXVirtualKeyboardService;
begin
  if TPlatformServices.Current.SupportsPlatformService(IFMXVirtualKeyboardService, IInterface(KeyboardService)) then
    KeyboardService.ShowVirtualKeyboard(Sender as TEdit);
end;

{$IFDEF IOS}
function TSGMainForm.GetDeviceModelString: String;
var
  Size: size_t;
  DeviceModelBuffer: array of Byte;
begin
  sysctlbyname('hw.machine', nil, @Size, nil, 0);

  if Size > 0 then
  begin
    SetLength(DeviceModelBuffer, Size);
    sysctlbyname('hw.machine', @DeviceModelBuffer[0], @Size, nil, 0);
    Result := UTF8ToString(MarshaledAString(DeviceModelBuffer));
  end
  else
    Result := EmptyStr;
end;
{$ENDIF}

procedure TSGMainForm.GetInfoDevice;
var
  LocaleService: IFMXLocaleService;
  OSVersion: TOSVersion;
begin
  FModelName := 'unknown';
{$IFDEF Android}
  FModelName := JStringToString(TJBuild.JavaClass.MODEL);
{$ENDIF}
{$IFDEF IOS}
  FModelName := Self.GetDeviceModelString;
{$ENDIF}

  FOSVersion := OSVersion.Name;
  FOSPlatform := Format('%d', [Ord(OSVersion.Platform)]);
  FOSMajMin := Format('%d.%d', [OSVersion.Major,OSVersion.Minor]);

  FOSLang := '';
  if TPlatformServices.Current.SupportsPlatformService(IFMXLocaleService, IInterface(LocaleService)) then
  begin
    FOSLang := LocaleService.GetCurrentLangID();
  end;

end;

end.

