unit uSGRest;

interface

uses
  System.Classes,
  System.SysUtils,
  IdBaseComponent, IdComponent,
  IdTCPConnection, IdTCPClient, IdHTTP;

type
  TClientRest = class
  private
    FServer: string;
    FPort: integer;
    function DoPost(aResource: string; aJSONBody: string; aHeaders: TStringList = nil): string;
    function DoPut(aResource: string; aJSONBody: string): string;
    procedure SetPort(const Value: integer);
    procedure SetServer(const Value: string);
  public
    constructor Create;
    function AuthorizeDevice(const aCode: string; const aDeviceId: string): string;
    function UpdateDevice(const aDeviceId: string; const aDeviceToken: string; const aDeviceDescription: string; aDeviceOs: string; aDeviceOsVersion: string; aDeviceType: string; aDeviceModel: string): string;
    function Login(const aUserName: string; const aPassword: string): string;
    property Server: string read FServer write SetServer;
    property Port: integer read FPort write SetPort;
  end;

const
  CONNECT_TIMEOUT = 10000;
  READ_TIMEOUT = 10000;

implementation

{ TClientRest }

function TClientRest.AuthorizeDevice(const aCode: string; const aDeviceId: string): string;
begin
  Result := DoPost('/device/authorize/'+aCode+'/'+aDeviceId,'');
end;

function TClientRest.UpdateDevice(const aDeviceId: string; const aDeviceToken: string; const aDeviceDescription: string; aDeviceOs: string; aDeviceOsVersion: string; aDeviceType: string; aDeviceModel: string): string;
begin
  Result := DoPut('/device/'+aDeviceId, Format('{"devicetoken":"%s", "description" : "%s", "devicetype": "%s", "deviceos": "%s", "deviceosversion": "%s", "devicemodel":"%s"}', [aDeviceToken,aDeviceDescription,aDeviceType,aDeviceModel,aDeviceOsVersion,aDeviceModel]));
end;

constructor TClientRest.Create;
begin
  inherited;
end;

function TClientRest.DoPost(aResource: string; aJSONBody: string; aHeaders: TStringList = nil): string;
var
  LHttp: TIdHttp;
  LBaseUrl: string;
  LUrl: string;
  LRequestStream: TStringStream;
  LResponseStream: TStringStream;
begin
  LBaseUrl := 'http://'+FServer+':'+FPort.ToString;
  LUrl := LBaseUrl + aResource;

  LHttp := TIdHttp.Create(nil);
  LHttp.ConnectTimeout := CONNECT_TIMEOUT;
  LHttp.ReadTimeout := READ_TIMEOUT;
  LRequestStream := TStringStream.Create(aJSONBody, TEncoding.UTF8);
  LResponseStream := TStringStream.Create;
  try
    LHttp.Request.ContentType := 'application/json';
    LHttp.Request.ContentEncoding := 'utf-8';

    if Assigned(aHeaders) then
    begin
      lHttp.Request.CustomHeaders.AddStrings(aHeaders);
    end;

    LHttp.Post(LUrl, LRequestStream, LResponseStream);
    Result := LResponseStream.DataString;
  finally
    if Assigned(aHeaders) then aHeaders.Free;
    LResponseStream.Free;
    LRequestStream.Free;
    LHttp.Free;
  end;
end;

function TClientRest.DoPut(aResource: string; aJSONBody: string): string;
var
  LHttp: TIdHttp;
  LBaseUrl: string;
  LUrl: string;
  LRequestStream: TStringStream;
  LResponseStream: TStringStream;
begin
  LBaseUrl := 'http://'+FServer+':'+FPort.ToString;
  LUrl := LBaseUrl + aResource;

  LHttp := TIdHttp.Create(nil);
  LHttp.ConnectTimeout := CONNECT_TIMEOUT;
  LHttp.ReadTimeout := READ_TIMEOUT;
  LRequestStream := TStringStream.Create(aJSONBody, TEncoding.UTF8);
  LResponseStream := TStringStream.Create;
  try
    LHttp.Request.ContentType := 'application/json';
    LHttp.Request.ContentEncoding := 'utf-8';

    LHttp.Put(LUrl, LRequestStream, LResponseStream);
    Result := LResponseStream.DataString;
  finally
    LResponseStream.Free;
    LRequestStream.Free;
    LHttp.Free;
  end;
end;

function TClientRest.Login(const aUserName, aPassword: string): string;
var
  LHeaders: TStringList;
begin
  // N.B. L'oggetto non deve essere distrutto perch viene distrutto dalla DoPost
  LHeaders := TStringList.Create;
  LHeaders.Add('jwtusername='+aUserName);
  LHeaders.Add('jwtpassword='+aPassword);
  Result := DoPost('/login', '', LHeaders);
end;

procedure TClientRest.SetPort(const Value: integer);
begin
  FPort := Value;
end;

procedure TClientRest.SetServer(const Value: string);
begin
  FServer := Value;
end;

end.
