unit Controller.Base;

interface

uses
  System.Classes,
  System.JSON,
  System.Generics.Collections,
  Data.Db,
  MVCFramework,
  MVCFramework.Commons,
  Server.Configuration,
  Server.DataModule,
  Service.Services;

type
  [MVCPath('/server')]
  TBaseController = class(TMVCController)
  strict private
    FDM: TDM;
  private
    FUseMockObjecsForDebug: boolean;
    FDevicesService: TDevicesService;
    procedure SetUseMockObjecsForDebug(const Value: boolean);
  protected
    function GetDataModule: TDM;
    function GetConfiguration: TConfiguration;
    function GetDevicesService: TDevicesService;
 public
    procedure MVCControllerAfterCreate; override;

    destructor Destroy; override;

    [MVCPath('/info')]
    [MVCHTTPMethod([httpGET])]
    procedure GetServerInfo;

    property Configuration: TConfiguration read GetConfiguration;
    property UseMockObjecsForDebug: boolean read FUseMockObjecsForDebug write SetUseMockObjecsForDebug;
  end;

implementation

uses
  System.SysUtils;

{ TBaseController }

destructor TBaseController.Destroy;
begin
  FDevicesService.Free;
  FDM.Free;
  inherited;
end;

function TBaseController.GetConfiguration: TConfiguration;
begin
  Result := GlobalConfiguration;
end;

function TBaseController.GetDataModule: TDM;
begin
  if not Assigned(FDM) then
  begin
    FDM := TDM.Create(nil, GetConfiguration);
    FDM.Init;
  end;
  Result := FDM;
end;

function TBaseController.GetDevicesService: TDevicesService;
begin
  if not Assigned(FDevicesService) then
    FDevicesService := TDevicesService.Create(GetDataModule);
  Result := FDevicesService;
end;

procedure TBaseController.GetServerInfo;
begin
  Render(200, Format('Produzione Server Rest Version %s del %s',['1.0.0','03/05/19']));
end;

//*------------------------------------------------------------------------------*
//* Il metodo viene richiamato dopo la creazione del controller da parte di DMVC *
//* Viene impostata la variabile che serve per generare i dati Fake per i test   *
//*------------------------------------------------------------------------------*
procedure TBaseController.MVCControllerAfterCreate;
begin
  inherited;
  // Assegna la connessione
  Self.GetDataModule;
  // Verifica se usare dati di test simulati
  FUseMockObjecsForDebug := GetConfiguration.MockObjects;
end;

procedure TBaseController.SetUseMockObjecsForDebug(const Value: boolean);
begin
  FUseMockObjecsForDebug := Value;
end;

end.
