program DemoServer;

{$APPTYPE CONSOLE}

uses
  System.SysUtils,
  MVCFramework.Logger,
  MVCFramework.Commons,
  MVCFramework.REPLCommandsHandlerU,
  Web.ReqMulti,
  Web.WebReq,
  Web.WebBroker,
  IdHTTPWebBrokerBridge,
  WebModule in 'WebModule.pas' {MyWebModule: TWebModule},
  Controller.Base in 'Controllers\Controller.Base.pas',
  Controller.Device in 'Controllers\Controller.Device.pas',
  System.Classes,
  LoggerProConfig in 'LoggerProConfig.pas',
  Errors in 'Errors.pas',
  Server.Configuration in 'Server\Server.Configuration.pas',
  Server.DataModule in 'Server\Server.DataModule.pas' {DM: TDataModule},
  Global.Utility in 'Global\Global.Utility.pas',
  Service.Services in 'Services\Service.Services.pas',
  OneSignal.REST in 'OneSignal\OneSignal.REST.pas',
  Server.Authentication in 'Server\Server.Authentication.pas',
  Model.BusinessObjects in '..\..\shared\Model\Model.BusinessObjects.pas';

{$R *.res}
{$STRONGLINKTYPES ON}

var
  LErrorMessage: string;
  LOutputScript: string;

procedure RunServer(APort: Integer);
var
  lServer: TIdHTTPWebBrokerBridge;
  lCustomHandler: TMVCCustomREPLCommandsHandler;
  lCmd: string;
begin
  Writeln('** DMVCFramework Server ** build ' + DMVCFRAMEWORK_VERSION);
  if ParamCount >= 1 then
    lCmd := ParamStr(1)
  else
    lCmd := 'start';

  lCustomHandler := function(const Value: String; const Server: TIdHTTPWebBrokerBridge; out Handled: Boolean): THandleCommandResult
    begin
      Handled := false;
      Result := THandleCommandResult.Unknown;
    end;

  lServer := TIdHTTPWebBrokerBridge.Create(nil);
  try
    lServer.DefaultPort := APort;

    { more info about MaxConnections
      http://www.indyproject.org/docsite/html/frames.html?frmname=topic&frmfile=TIdCustomTCPServer_MaxConnections.html }
    lServer.MaxConnections := 0;

    { more info about ListenQueue
      http://www.indyproject.org/docsite/html/frames.html?frmname=topic&frmfile=TIdCustomTCPServer_ListenQueue.html }
    lServer.ListenQueue := 200;

    Writeln('Write "quit" or "exit" to shutdown the server');
    repeat
      if lCmd.IsEmpty then
      begin
        Write('-> ');
        ReadLn(lCmd)
      end;
      try
        case HandleCommand(lCmd.ToLower, lServer, lCustomHandler) of
          THandleCommandResult.Continue:
            begin
              Continue;
            end;
          THandleCommandResult.Break:
            begin
              Break;
            end;
          THandleCommandResult.Unknown:
            begin
              REPLEmit('Unknown command: ' + lCmd);
            end;
        end;
      finally
        lCmd := '';
      end;
    until false;

  finally
    lServer.Free;
  end;
end;

begin
  LOutputScript := '';
  LErrorMessage := '';
  ReportMemoryLeaksOnShutdown := True;
  GlobalConfiguration := TConfiguration.GetInstance;

  try
//
    try
      IsMultiThread := True;
      if WebRequestHandler <> nil then
        WebRequestHandler.WebModuleClass := WebModuleClass;
      WebRequestHandlerProc.MaxConnections := 1024;
      RunServer(8080);
    except
      on E: Exception do
        Writeln(E.ClassName, ': ', E.Message);
    end;

  finally
    if Assigned(GlobalConfiguration) then GlobalConfiguration.Free;
  end;
end.
