unit Global.Utility;

interface

uses
  System.SysUtils,
  System.UITypes,
  System.Generics.Collections,
  VCL.Dialogs;

type
  TUtility = class
  private
    class function Config(const aSection, aIdent: String; aValue: String = ''; aGet: Boolean = true): String;
    class procedure CheckConfigFile(const aFile: String);
    class function GetConfigFile: String;
  public
    class function GetConfig(const aSection, aIdent: String): String;
    class procedure SetConfig(const aSection, aIdent, aValue: String);
    class function ConfigFileExist: Boolean;
    class function GetMSSQLFormattedDateTime(const aDate: TDate; const aTime: TTime = 0; const aOnlyDate: Boolean = false): String;
    class function ExtractString(aValue: string; aOccurrence: integer; aSeparator: Char; aToEnd: Boolean): string;
    class function GetFileNameWithGuidAndTimeStamp(AFileNameWithoutExtension: string): string;
    class function GetSiNo(aValue: Boolean): string;
    class function GetProdVersion(const aExeName: string): string;
    class function IsClientToUpdate(const aClientVersion: String): Boolean;
    class procedure Error(AText: string);
  end;

implementation

uses
  System.IniFiles, System.IOUtils, System.Notification;

{ TUtility }

class function TUtility.GetFileNameWithGuidAndTimeStamp(AFileNameWithoutExtension: string): string;
var
  LGuid: TGuid;
  LGuidStr: string;
begin
  LGuid := TGuid.NewGuid;
  LGuidStr := StringReplace(LGuid.ToString, '{', '', [rfReplaceAll]);
  LGuidStr := StringReplace(LGuidStr, '}', '', [rfReplaceAll]);
  Result := AFileNameWithoutExtension + '_' + FormatDateTime('yyyymmddhhnnss', Now()) + '_' + LGuidStr;
end;

class procedure TUtility.CheckConfigFile(const aFile: String);
var
  lHandle: THandle;
begin
  if not FileExists(aFile) then
  begin
    lHandle := FileCreate(aFile);
    FileClose(lHandle);
  end;
end;

class function TUtility.Config(const aSection, aIdent: String; aValue: String; aGet: Boolean): String;
var
  lIni: TiniFile;
  lFile: String;
begin
  lFile := GetConfigFile;
  CheckConfigFile(lFile);

  lIni := TiniFile.Create(lFile);
  try
    if aGet then
      Result := lIni.ReadString(aSection, aIdent, '')
    else
      lIni.WriteString(aSection, aIdent, aValue);
  finally
    FreeAndNil(lIni);
  end;
end;

class function TUtility.ConfigFileExist: Boolean;
begin
  Result := FileExists(GetConfigFile);
end;

class procedure TUtility.Error(AText: string);
begin
  MessageDlg(AText,mtError,[mbOk],0);
end;

class function TUtility.ExtractString(aValue: string; aOccurrence: integer; aSeparator: Char; aToEnd: Boolean): string;
var
  i, N: integer;
begin
  Result := '';
  N := 0;

  for i := 1 to length(aValue) do
  begin
    if (((N = aOccurrence - 1) and not aToEnd) or ((N >= aOccurrence - 1) and aToEnd)) and ((aValue[i] <> aSeparator) or aToEnd) then
      Result := Result + aValue[i];
    if aValue[i] = aSeparator then
      N := N + 1;
    if (N > aOccurrence - 1) and not aToEnd then
      break;
  end;
end;

class function TUtility.GetConfig(const aSection, aIdent: String): String;
begin
  Result := Config(aSection, aIdent);
end;

class function TUtility.GetConfigFile: String;
begin
  Result := TPath.combine(ExtractFilePath(ParamStr(0)), 'config.ini');
end;

class function TUtility.GetMSSQLFormattedDateTime(const aDate: TDate; const aTime: TTime; const aOnlyDate: Boolean): String;
const
  cTime = ' hh:nn:ss';
  cDate = 'yyyymmdd';
var
  lDateTime: String;
begin
  lDateTime := cDate + cTime;
  if aOnlyDate then
    lDateTime := cDate;

  Result := FormatDateTime(lDateTime, aDate + aTime);
end;

class function TUtility.GetProdVersion(const aExeName: string): string;
var
  lMajor, lMinor, lBuilt: Cardinal;
begin
  Result := '';
end;

class function TUtility.GetSiNo(aValue: Boolean): string;
begin
  if aValue then
    Result := 'SI'
  else
    Result := 'NO';
end;

class function TUtility.IsClientToUpdate(const aClientVersion: String): Boolean;
var
  lPath: String;
begin
  lPath := TPath.Combine(TUtility.GetConfig('PATHS', 'Client'), TUtility.GetConfig('PATHS', 'ClientName'));
  Result := GetProdVersion(lPath) <> aClientVersion;
end;

class procedure TUtility.SetConfig(const aSection, aIdent, aValue: String);
begin
  Config(aSection, aIdent, aValue, false);
end;

end.
