unit OneSignal.REST;

interface

uses
  System.SysUtils,
  System.Classes,
  System.JSON,
  IdIOHandler,
  IdIOHandlerSocket, IdIOHandlerStack, IdSSL, IdSSLOpenSSL, IdBaseComponent,
  IdComponent, IdTCPConnection, IdTCPClient, IdHTTP,
  IdSSLOpenSSLHeaders;

type
  TOneSignal = class
  private
  public
    class function UpdatePlayer(const aAppId: string; const aAPIKey: string; const aDeviceToken, aExternalId, aDeviceType, aDeviceModel, aDeviceOS: string): string;
    class function SendPushToPlayer(const aAppId, aAPIKey,
  aExternalId: string): boolean;
  end;

implementation

{ TOneSignal }

class function TOneSignal.SendPushToPlayer(const aAppId, aAPIKey,
  aExternalId: string): boolean;
var
  LIdHttp: TIdHttp;
  LIdIOHandler: TIdSSLIOHandlerSocketOpenSSL;
  jsonString: string;
  JsonToSend: TStringStream;
  response: string;
  LJSONObject: TJSONObject;
  LSuccess: Boolean;
  LExternalId: string;
begin
  LIdHTTP := TIdHTTP.Create(nil);
  try
    LIdIOHandler := TIdSSLIOHandlerSocketOpenSSL.Create(LIdHTTP);
    LIdIOHandler.SSLOptions.Method := sslvSSLv23;
    LIdHTTP.IOHandler := LIdIOHandler;

    LIdHTTP.Request.ContentType := 'application/json';
    LIdHTTP.Request.CustomHeaders.AddValue('Authorization','Basic '+aAPIKey);
    LIdHTTP.Request.Charset := 'UTF-8';

    jsonString := '{"app_id" : "'+aAppId+'",'+
                  '"include_player_ids":["'+aExternalId+'"],'+
                  '"contents": {"en": "Push notifications con DELPHI e ONESIGNAL"},'+
                  '"headings": {"en": "DELPHIDAY2019 - MARCO MOTTADELLI"}}';

    JsonToSend := TStringStream.Create(jsonString, TEncoding.UTF8);
    try
      LSuccess := False;

      // Risposta OK
      {
          "success": true,
          "id": "757b1e38-028b-420a-af3d-3ed649babdc7"
      }
      // Risposta con ERRORI
      {
          "errors": [
              "app_id not found. You may be missing a Content-Type: application/json header."
          ]
      }
      response := LIdHTTP.Post('https://onesignal.com/api/v1/notifications', JsonToSend);

      LJSONObject := TJSONObject.ParseJSONValue(response) as TJSONObject;
      try
        LJSONObject.TryGetValue<string>('id', LExternalId);

        if not LExternalId.IsEmpty then
          Result := True
        else
          Result := False;
          //raise Exception.Create('Error in Send Push Notification To Device By OneSignal');

      finally
        LJSONObject.Free;
      end;
    except
      on E: EIdHTTPProtocolException do
        response := e.ErrorMessage;
    end;

  finally
    LIdHTTP.Free;
  end;
end;

class function TOneSignal.UpdatePlayer(const aAppId: string; const aAPIKey: string; const aDeviceToken,
  aExternalId, aDeviceType, aDeviceModel, aDeviceOS: string): string;
var
  LIdHttp: TIdHttp;
  LIdIOHandler: TIdSSLIOHandlerSocketOpenSSL;
  jsonString: string;
  JsonToSend: TStringStream;
  response: string;
  LJSONObject: TJSONObject;
  LSuccess: Boolean;
  LExternalId: string;
begin
  LIdHTTP := TIdHTTP.Create(nil);
  try
    LIdIOHandler := TIdSSLIOHandlerSocketOpenSSL.Create(LIdHTTP);
    LIdIOHandler.SSLOptions.Method := sslvSSLv23;
    LIdHTTP.IOHandler := LIdIOHandler;

    LIdHTTP.Request.ContentType := 'application/json';
    LIdHTTP.Request.CustomHeaders.AddValue('Authorization','Basic '+aAPIKey);
    LIdHTTP.Request.Charset := 'UTF-8';

    jsonString := '{"app_id" : "'+aAppId+'",'+
                  '"identifier":"'+aDeviceToken+'",'+
                  '"language":"en",'+
                  '"timezone":-28800,'+
                  '"game_version":"1.0",'+
                  '"device_os":"'+ADeviceModel+'",'+
                  '"device_type":'+aDeviceType+','+  // 0=ios, 1=android
                  '"test_type":1,'+                  // 1 = Development TODO: Rimuovere per produzione!!!!!
                  '"device_model":"'+ADeviceOS+'"}';

    JsonToSend := TStringStream.Create(jsonString, TEncoding.UTF8);
    try
      LSuccess := False;

      // Risposta OK
      {
          "success": true,
          "id": "757b1e38-028b-420a-af3d-3ed649babdc7"
      }
      // Risposta con ERRORI
      {
          "errors": [
              "app_id not found. You may be missing a Content-Type: application/json header."
          ]
      }
      response := LIdHTTP.Post('https://onesignal.com/api/v1/players', JsonToSend);

      LJSONObject := TJSONObject.ParseJSONValue(response) as TJSONObject;
      try
        LJSONObject.TryGetValue<boolean>('success', LSuccess);
        LJSONObject.TryGetValue<string>('id', LExternalId);

        if LSuccess then
          Result := LExternalId
        else
          raise Exception.Create('Error in Synchronization Device On OneSignal');

      finally
        LJSONObject.Free;
      end;
    except
      on E: EIdHTTPProtocolException do
        response := e.ErrorMessage;
    end;

  finally
    LIdHTTP.Free;
  end;
end;

end.
