unit Server.Configuration;

interface

uses
  System.SysUtils,
  System.IniFiles,
  Data.DB,
  FireDAC.Comp.Client;

const
  REST_SERVER_INI_SECTION = 'Server_Parameters';
  REST_SERVER_ONESIGNAL_INI_SECTION = 'OneSignal_Parameters';

type
  TConfiguration = class(TObject)
  private
    FLogActive: boolean;
    FSQLUser_Name: string;
    FSQLDatabase: string;
    FSQLDriverID: string;
    FSQLPassword: string;
    FSQLServer: string;
    FSQLMetaDefSchema: string;
    FMockObjects: boolean;
    FAppIdPushServer: string;
    FAPIKeyPushServer: string;
    FSQLPort: integer;

    procedure SetLogActive(const Value: boolean);
    class var FInstance: TConfiguration;
    procedure SetSQLDatabase(const Value: string);
    procedure SetSQLDriverID(const Value: string);
    procedure SetSQLMetaDefSchema(const Value: string);
    procedure SetSQLPassword(const Value: string);
    procedure SetSQLServer(const Value: string);
    procedure SetSQLUser_Name(const Value: string);
    procedure SetMockObjects(const Value: boolean);
    procedure SetAppIdPushServer(const Value: string);
    procedure SetAPIKeyPushServer(const Value: string);
    procedure SetSQLPort(const Value: integer);
  public
    destructor Destroy; override;
    class function GetInstance: TConfiguration;
    procedure ReadConfiguration;
    property LogActive: boolean read FLogActive write SetLogActive;
    property MockObjects: boolean read FMockObjects write SetMockObjects;
    property SQLDriverID: string read FSQLDriverID write SetSQLDriverID;
    property SQLServer: string read FSQLServer write SetSQLServer;
    property SQLDatabase: string read FSQLDatabase write SetSQLDatabase;
    property SQLUser_Name: string read FSQLUser_Name write SetSQLUser_Name;
    property SQLPassword: string read FSQLPassword write SetSQLPassword;
    property SQLPort: integer read FSQLPort write SetSQLPort;
    property SQLMetaDefSchema: string read FSQLMetaDefSchema write SetSQLMetaDefSchema;
    property AppIdPushServer: string read FAppIdPushServer write SetAppIdPushServer;
    property APIKeyPushServer: string read FAPIKeyPushServer write SetAPIKeyPushServer;
  end;

var
  GlobalConfiguration: TConfiguration;

implementation

uses
  Global.Utility;

destructor TConfiguration.Destroy;
begin
  inherited;
end;


class function TConfiguration.GetInstance: TConfiguration;
begin
  if not Assigned(FInstance) then
  begin
    FInstance := TConfiguration.Create;
    FInstance.ReadConfiguration;
  end;
  Result := FInstance
end;

procedure TConfiguration.ReadConfiguration;
var
  LConfigurationFile: string;
  LIniFile: TIniFile;
begin
  LConfigurationFile := '.\config.ini';
  //*----------------------------------*
  //* Check Service Configuration file *
  //*----------------------------------*
  if not FileExists(LConfigurationFile) then
    raise Exception.Create('Configuration File is missing (Config.ini)');
  //*---------------------------------*
  //* Read Service Configuration file *
  //*---------------------------------*
  LIniFile := TIniFile.Create(LConfigurationFile);
  try
    // Server info
    FLogActive := LIniFile.ReadBool(REST_SERVER_INI_SECTION,'LogActive',False);
    FMockObjects := LIniFile.ReadBool(REST_SERVER_INI_SECTION,'MockObjects',False);
    FSQLUser_Name := LIniFile.ReadString(REST_SERVER_INI_SECTION,'User_Name','');
    FSQLDatabase := LIniFile.ReadString(REST_SERVER_INI_SECTION,'Database','');
    FSQLDriverID := LIniFile.ReadString(REST_SERVER_INI_SECTION,'DriverID','');
    // Decode md5 password
    FSQLPassword := LIniFile.ReadString(REST_SERVER_INI_SECTION,'Password','');
    FSQLServer := LIniFile.ReadString(REST_SERVER_INI_SECTION,'Server','');
    FSQLPort := LIniFile.ReadInteger(REST_SERVER_INI_SECTION,'Port',3306);
    FSQLMetaDefSchema := LIniFile.ReadString(REST_SERVER_INI_SECTION,'MetaDefSchema','');
    // One Signal info
    FAppIdPushServer := LIniFile.ReadString(REST_SERVER_ONESIGNAL_INI_SECTION,'AppIdPushServer','');
    FAPIKeyPushServer := LIniFile.ReadString(REST_SERVER_ONESIGNAL_INI_SECTION,'APIKeyPushServer','');
  finally
    LIniFile.Free;
  end;
  //*----------------------------------*
  //* Check Minimal Configuration info *
  //*----------------------------------*
  if FSQLServer.IsEmpty then
    raise Exception.Create('Server String is missing in (Config.ini)');
  if FSQLDriverID.IsEmpty then
    raise Exception.Create('DriverID String is missing in (Config.ini)');

end;

procedure TConfiguration.SetAPIKeyPushServer(const Value: string);
begin
  FAPIKeyPushServer := Value;
end;

procedure TConfiguration.SetAppIdPushServer(const Value: string);
begin
  FAppIdPushServer := Value;
end;

procedure TConfiguration.SetLogActive(const Value: boolean);
begin
  FLogActive := Value;
end;

procedure TConfiguration.SetMockObjects(const Value: boolean);
begin
  FMockObjects := Value;
end;

procedure TConfiguration.SetSQLDatabase(const Value: string);
begin
  FSQLDatabase := Value;
end;

procedure TConfiguration.SetSQLDriverID(const Value: string);
begin
  FSQLDriverID := Value;
end;

procedure TConfiguration.SetSQLMetaDefSchema(const Value: string);
begin
  FSQLMetaDefSchema := Value;
end;

procedure TConfiguration.SetSQLPassword(const Value: string);
begin
  FSQLPassword := Value;
end;

procedure TConfiguration.SetSQLPort(const Value: integer);
begin
  FSQLPort := Value;
end;

procedure TConfiguration.SetSQLServer(const Value: string);
begin
  FSQLServer := Value;
end;

procedure TConfiguration.SetSQLUser_Name(const Value: string);
begin
  FSQLUser_Name := Value;
end;

end.

