unit Server.DataModule;

interface

uses
  System.SysUtils,
  System.Classes,
  System.Generics.Collections,
  Data.DB,
  FireDAC.UI.Intf,
  FireDAC.VCLUI.Wait,
  FireDAC.Stan.Intf,
  FireDAC.Stan.Option,
  FireDAC.Stan.Error,
  FireDAC.Phys.Intf,
  FireDAC.Stan.Def,
  FireDAC.Stan.Pool,
  FireDAC.Stan.Async,
  FireDAC.Phys,
  FireDAC.Comp.Client,
  FireDAC.Comp.UI,
  FireDAC.Phys.MSSQLDef,
  FireDAC.Phys.ODBCBase,
  FireDAC.Phys.MSSQL,
  FireDAC.DApt,
  FireDAC.Stan.Param,
  FireDAC.DatS,
  FireDAC.DApt.Intf,
  FireDAC.Comp.DataSet,
  Server.Configuration,
  FireDAC.Stan.StorageBin,
  FireDAC.Stan.StorageJSON, FireDAC.Phys.MySQLDef, FireDAC.Phys.MySQL;

type
  TDM = class(TDataModule)
    FDGUIxWaitCursor1: TFDGUIxWaitCursor;
    FDPhysMSSQLDriverLink1: TFDPhysMSSQLDriverLink;
    FDConnection1: TFDConnection;
    dsDevices: TFDQuery;
    updDevices: TFDUpdateSQL;
    FDPhysMySQLDriverLink1: TFDPhysMySQLDriverLink;
  private
    FConfiguration: TConfiguration;
    FConnection: TFDConnection;
    FTransaction: TFDTransaction;
  public
    constructor Create(AOwner: TComponent; AConfiguration: TConfiguration);
    destructor Destroy; override;
    procedure Init;
    procedure CommitTransaction;
    procedure RollbackTransaction;
    procedure BeginTransaction;
    function  GetConnection: TFDConnection;
    function  NewQuery: TFDQuery;
    procedure ReleaseQuery(AQuery: TFDQuery);
    function  NewUpdateSql: TFDUpdateSql;
    procedure ReleaseUpdateSql(AUpdateSql: TFDUpdateSql);
    property Configuration: TConfiguration read FConfiguration;
  end;

implementation

{%CLASSGROUP 'Vcl.Controls.TControl'}

{$R *.dfm}


function TDM.NewUpdateSql: TFDUpdateSql;
begin
  Result := TFDUpdateSql.Create(nil);
  Result.Connection := FConnection;
end;

procedure TDM.ReleaseUpdateSql(AUpdateSql: TFDUpdateSql);
begin
  if Assigned(AUpdateSql) then
  begin
    AUpdateSql.Free;
  end;
end;

procedure TDM.BeginTransaction;
begin
  FConnection.Transaction.StartTransaction;
end;

procedure TDM.CommitTransaction;
begin
  FConnection.Transaction.Commit;
end;

constructor TDM.Create(AOwner: TComponent; AConfiguration: TConfiguration);
begin
  inherited Create(AOwner);
  FConnection := TFDConnection.Create(nil);
  FTransaction := TFDTransaction.Create(nil);
  FConnection.Transaction := FTransaction;
  FConfiguration := AConfiguration;
end;

destructor TDM.Destroy;
begin
  if Assigned(FTransaction) then FTransaction.Free;
  if Assigned(FConnection) then FConnection.Free;
  inherited;
end;

function TDM.GetConnection: TFDConnection;
begin
  Result := FConnection;
end;

procedure TDM.Init;
begin
  FConnection.Params.Values['DriverID'] := FConfiguration.SQLDriverID;
  FConnection.Params.Values['Server'] := FConfiguration.SQLServer;
  FConnection.Params.Values['Database'] := FConfiguration.SQLDatabase;
  FConnection.Params.Values['User_Name'] := FConfiguration.SQLUser_Name;
  FConnection.Params.Values['Password'] := FConfiguration.SQLPassword;
  FConnection.Params.Values['MetaDefSchema'] := FConfiguration.SQLMetaDefSchema;
  FConnection.OptionsIntf.FormatOptions.OwnMapRules := True;
  FConnection.OptionsIntf.FormatOptions.MapRules.Clear;

  // Change MSSQL DataType Native FireDAC to DateTime Delphi
  with FConnection.OptionsIntf.FormatOptions.MapRules.Add do
  begin
    SourceDataType := dtDateTimeStamp;
    TargetDataType := dtDateTime;
  end;

  // Assegna la connessione ai componenti
  updDevices.Connection := FConnection;
  dsDevices.Connection := FConnection;

end;


function TDM.NewQuery: TFDQuery;
begin
  Result := TFDQuery.Create(nil);
  Result.Connection := FConnection;
end;

procedure TDM.ReleaseQuery(AQuery: TFDQuery);
begin
  if Assigned(AQuery) then
  begin
    AQuery.Close;
    AQuery.Free;
  end;
end;

procedure TDM.RollbackTransaction;
begin
  FConnection.Rollback;
end;

end.
