unit WebModule;

interface

uses
  System.SysUtils,
  System.Classes,
  Web.HTTPApp,
  MVCFramework,
  Server.Configuration,
  Controller.Base,
  Controller.Device;

type
  TMyWebModule = class(TWebModule)
    procedure WebModuleCreate(Sender: TObject);
    procedure WebModuleDestroy(Sender: TObject);
  private
    FMVC: TMVCEngine;
  public
  end;

var
  WebModuleClass: TComponentClass = TMyWebModule;
  
implementation

{$R *.dfm}

uses
  System.IOUtils,
  System.DateUtils,
  MVCFramework.Commons,
  MVCFramework.Middleware.JWT,
  MVCFramework.JWT,

  Server.Authentication;

procedure TMyWebModule.WebModuleCreate(Sender: TObject);
var
  lClaimsSetup: TJWTClaimsSetup;
begin
  lClaimsSetup := procedure(const JWT: TJWT)
    begin
      JWT.Claims.Issuer := 'DEMO MVC Framework';
      // N.B. Il token non scade mai
      JWT.Claims.NotBefore := EncodeDateTime(2018,01,01,23,59,59,0); // Now - OneMinute * 60; // valid since 60 minutes ago
      JWT.Claims.IssuedAt := Now;
      // N.B. Il token scadr nel 2099
      JWT.Claims.ExpirationTime := EncodeDateTime(2099,01,01,23,59,59,0);     // Now + OneMinute * 60;
      // JWT.CustomClaims['mycustomvalue'] := 'hello there';
      // Here we dont use a fixed ExpirationTime but a LiveValidityWindowInSeconds
      // to make the ExpirationTime dynamic, incrementing the
      // ExpirationTime by LiveValidityWindowInSeconds seconds at each request
      JWT.LiveValidityWindowInSeconds := 60 * 60; // 1 hour
    end;

  FMVC := TMVCEngine.Create(Self,
    procedure(Config: TMVCConfig)
    begin
      // enable static files
      Config[TMVCConfigKey.DocumentRoot] := TPath.Combine(ExtractFilePath(GetModuleName(HInstance)), 'www');
      // session timeout (0 means session cookie)
      Config[TMVCConfigKey.SessionTimeout] := '0';
      // default content-type
      Config[TMVCConfigKey.DefaultContentType] := TMVCConstants.DEFAULT_CONTENT_TYPE;
      // default content charset
      Config[TMVCConfigKey.DefaultContentCharset] := TMVCConstants.DEFAULT_CONTENT_CHARSET;
      // unhandled actions are permitted?
      Config[TMVCConfigKey.AllowUnhandledAction] := 'false';
      // default view file extension
      Config[TMVCConfigKey.DefaultViewFileExtension] := 'html';
      // view path
      Config[TMVCConfigKey.ViewPath] := 'templates';
      // Enable Server Signature in response
      Config[TMVCConfigKey.ExposeServerSignature] := 'true';
      // Define a default URL for requests that don't map to a route or a file (useful for client side web app)
      Config[TMVCConfigKey.FallbackResource] := 'index.html';
    end);

  // Aggancia il controller all'engine
  FMVC.AddController(TBaseController)
      .AddController(TDeviceController);
end;

procedure TMyWebModule.WebModuleDestroy(Sender: TObject);
begin
  FMVC.Free;
end;

end.
